@@ -99,7 +99,9 @@ SPALeRASGD<DecayPolicyType>::Optimize(
   ElemType lastObjective = DBL_MAX;
 
   // Controls early termination of the optimization process.
-  bool terminate = Callback::BeginOptimization(*this, f, iterate, callbacks...);
+  bool terminate = false;
+
+  Callback::BeginOptimization(*this, f, iterate, callbacks...);
 
   // Calculate the first objective function.
   for (size_t i = 0; i < numFunctions; i += batchSize)
@@ -108,7 +110,7 @@ SPALeRASGD<DecayPolicyType>::Optimize(
     const ElemType objective = f.Evaluate(iterate, i, effectiveBatchSize);
     overallObjective += objective;
 
-    Callback::Evaluate(*this, f, iterate, objective, callbacks...);
+    terminate |= Callback::Evaluate(*this, f, iterate, objective, callbacks...);
   }
 
   ElemType currentObjective = overallObjective / numFunctions;
@@ -136,7 +138,7 @@ SPALeRASGD<DecayPolicyType>::Optimize(
   BaseGradType gradient(iterate.n_rows, iterate.n_cols);
   const size_t actualMaxIterations = (maxIterations == 0) ?
       std::numeric_limits<size_t>::max() : maxIterations;
-  Callback::BeginEpoch(*this, f, iterate, epoch, overallObjective,
+  terminate |= Callback::BeginEpoch(*this, f, iterate, epoch, overallObjective,
       callbacks...);
   for (size_t i = 0; i < actualMaxIterations && !terminate;
       /* incrementing done manually */)
@@ -157,6 +159,8 @@ SPALeRASGD<DecayPolicyType>::Optimize(
 
     terminate |= Callback::EvaluateWithGradient(*this, f, iterate,
         currentObjective, gradient, callbacks...);
+    if (terminate)
+      break;
 
     // Use the update policy to take a step.
     if (!instUpdatePolicy.As<InstUpdatePolicyType>().Update(stepSize,
@@ -199,9 +203,7 @@ SPALeRASGD<DecayPolicyType>::Optimize(
         return overallObjective;
       }
 
-      if (std::abs(lastObjective - overallObjective) < tolerance ||
-          Callback::BeginEpoch(*this, f, iterate, epoch, overallObjective,
-              callbacks...))
+      if (std::abs(lastObjective - overallObjective) < tolerance)
       {
         Info << "SPALeRA SGD: minimized within tolerance " << tolerance
             << "; terminating optimization." << std::endl;
@@ -215,6 +217,9 @@ SPALeRASGD<DecayPolicyType>::Optimize(
       overallObjective = 0;
       currentFunction = 0;
 
+      terminate |= Callback::BeginEpoch(*this, f, iterate, epoch,
+          overallObjective, callbacks...);
+
       if (shuffle) // Determine order of visitation.
         f.Shuffle();
     }
@@ -233,7 +238,9 @@ SPALeRASGD<DecayPolicyType>::Optimize(
       const ElemType objective = f.Evaluate(iterate, i, effectiveBatchSize);
       overallObjective += objective;
 
-      Callback::Evaluate(*this, f, iterate, objective, callbacks...);
+      // The optimization is over, so we don't need to care about the result of
+      // the callback.
+      (void) Callback::Evaluate(*this, f, iterate, objective, callbacks...);
     }
   }
 
